"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readU8aU32 = readU8aU32;
exports.writeU8aU32 = writeU8aU32;
exports.modifyHdr = modifyHdr;
exports.modifyKey = modifyKey;
exports.newHdr = newHdr;
exports.newKey = newKey;
exports.parseHdr = parseHdr;
exports.parseKey = parseKey;
exports.serializeKey = serializeKey;

var _utilCrypto = require("@polkadot/util-crypto");

var _defaults = require("./defaults");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
function readU8aU32(u8a, offset) {
  // reverse the writing, highest bits goes first, lowest are last
  return (u8a[offset] << 24) + (u8a[offset + 1] << 16) + (u8a[offset + 2] << 8) + u8a[offset + 3];
}

function writeU8aU32(u8a, value, offset) {
  // write the highest bits first - this way, when flagged, we only need to
  // read a single byte (this is useful in the cases where we re-encode a trie)
  u8a[offset] = value >> 24 & 0xff;
  u8a[offset + 1] = value >> 16 & 0xff;
  u8a[offset + 2] = value >> 8 & 0xff;
  u8a[offset + 3] = value & 0xff;
}

function modifyHdr(hdr, hdrIndex, linkAt, isKey) {
  writeU8aU32(hdr, isKey ? linkAt | _defaults.BITS_F : linkAt, hdrIndex * _defaults.HDR_ENTRY_SIZE);
  return hdr;
}

function modifyKey(keyData, valAt, valSize) {
  writeU8aU32(keyData, valAt, _defaults.KEY_DATA_SIZE);
  writeU8aU32(keyData, valSize, _defaults.KEY_DATA_SIZE + _defaults.U32_SIZE);
  return keyData;
}

function newHdr(indexes) {
  const hdr = new Uint8Array(_defaults.HDR_TOTAL_SIZE);
  indexes.forEach((_ref) => {
    let {
      dataAt,
      hdrIndex,
      isKey
    } = _ref;
    return modifyHdr(hdr, hdrIndex, dataAt, isKey);
  });
  return hdr;
}

function newKey(key, _ref2) {
  let {
    valAt,
    valSize
  } = _ref2;
  const keyData = new Uint8Array(_defaults.KEY_TOTAL_SIZE);
  keyData.set(key.buffer, 0);
  return modifyKey(keyData, valAt, valSize);
}

function parseHdr(hdr, hdrIndex) {
  const u32 = readU8aU32(hdr, hdrIndex * _defaults.HDR_ENTRY_SIZE);
  const linkTo = u32 & _defaults.BITS_U;
  return {
    isKey: linkTo !== u32,
    linkTo
  };
}

function parseKey(keyData) {
  return {
    valAt: readU8aU32(keyData, _defaults.KEY_DATA_SIZE),
    valSize: readU8aU32(keyData, _defaults.KEY_DATA_SIZE + _defaults.U32_SIZE)
  };
}

function serializeKey(u8a) {
  // Convert any non-32-byte keys into a hash of the key. This allows for proper
  // key distribution. In practice, the inputs should already be hashed, in the
  // case of using a trie, however if used directly, this would come into play
  const buffer = u8a.length === _defaults.KEY_DATA_SIZE ? u8a : (0, _utilCrypto.blake2AsU8a)(u8a);
  const parts = new Uint8Array(_defaults.KEY_PARTS_SIZE);
  let offset = 0;

  for (let i = 0; i < _defaults.KEY_DATA_SIZE; i++) {
    const item = buffer[i];
    parts[offset] = item & 0b1111;
    parts[offset + 1] = item >> 4;
    offset += 2;
  }

  return {
    buffer,
    parts
  };
}